Attribute VB_Name = "APIs"
Option Explicit
'======================================================================
'   Informacin del Archivo
'   ------------------------
'
'   Name    : Apis.bas
'   Author  : Jota Desarrollo de Software
'   Date    : 04-Feb-2000       Last Update : 02-Jul-2003
'
'   Notes   : Contiene las declaraciones de funciones, constantes y
'             tipos necesarios para realizar llamadas a las APIs de
'             Windows.
'
'             La idea bsica para la creacin de estas funciones se
'             obtuvo principalmente de:
'             API Guide, del grupo KDE (http://users.turboline.be/btl10148/index.html)
'             VBNet (www.mvps.org/vbnet)
'
'             El cdigo fue adaptado, modificado y enriquecido. Si alguna porcin
'             de cdigo se transcribiera textualmente, se aclarar su procedencia
'             en el encabezado.
'
'             Hechas las salvedades, el cdigo de este archivo se considera
'             de LIBRE DISTRIBUCIN, citando la fuente.
'======================================================================

'===============================================================================
'Declaracin de Constantes
'===============================================================================

'Constantes para SendMessage
'autobuscar en una lista
Public Const LB_FINDSTRING = &H18F

'Constante para obtener el nombre de la PC
Public Const MAX_COMPUTERNAME_LENGTH = 255

'Constantes para BrowseForFolder
Const BIF_RETURNONLYFSDIRS = 1
Const WM_USER = &H400&
'Constantes para Mensajes recibidos por BrowseForFolder
Const BFFM_INITIALIZED = 1
Const BFFM_SETSTATUSTEXT = (WM_USER + 100)
Const BFFM_ENABLEOK = (WM_USER + 101)
Const BFFM_SETSELECTION = (WM_USER + 102)


'Constantes para SHGetFileInfo y BrowseForFolder
Const SHGFI_DISPLAYNAME = &H200
Const SHGFI_TYPENAME = &H400
Const MAX_PATH = 260

'===============================================================================
'Definiciones de Tipo
'===============================================================================

'Definicin de tipo para BrowseForFolder
Private Type BrowseInfo
    hwndOwner As Long
    pIDLRoot As Long
    pszDisplayName As Long
    lpszTitle As Long
    ulFlags As Long
    lpfnCallback As Long
    lParam As Long
    iImage As Long
End Type


'===============================================================================
'Declaracin de variables
'===============================================================================

Dim strFolderIni As String      'Variable de intercambio entre BrowseForFolder y su CallBackProc


'===============================================================================
'Declaracin de Funciones API
'
'Las funciones declaradas PUBLIC se invocan directamente desde el cdigo.
'Las PRIVATE son llamada en este mismo archivo por una funcin intermedia
'por otra funcin API.
'===============================================================================


'Ejecutar archivos y documentos
Public Declare Function ShellExecute Lib "shell32" Alias "ShellExecuteA" (ByVal hWnd As Long, ByVal lpOperation As String, ByVal lpFile As String, ByVal lpParameters As String, ByVal lpDitectory As String, ByVal nShowCmd As Long) As Long

'Obtener las unidades de disco disponibles
Private Declare Function GetLogicalDrives Lib "kernel32" () As Long

'Obtener el tipo de unidad
Private Declare Function GetDriveType Lib "kernel32" Alias "GetDriveTypeA" (ByVal nDrive As String) As Long

'Subclasificar un control
Public Declare Function SendMessage Lib "user32.dll" Alias "SendMessageA" (ByVal hWnd As Long, ByVal wMsg As Long, ByVal wParam As Long, lParam As Any) As Long

'Obtener cadena de caracteres de un archivo INI
Declare Function GetPrivateProfileString Lib "kernel32" Alias "GetPrivateProfileStringA" (ByVal lpApplicationName As String, ByVal lpKeyName As Any, ByVal lpDefault As String, ByVal lpReturnedString As String, ByVal nSize As Long, ByVal lpFileName As String) As Long

'Escribir a un archivo INI
Declare Function WritePrivateProfileString Lib "kernel32" Alias "WritePrivateProfileStringA" (ByVal lpApplicationName As String, ByVal lpKeyName As Any, ByVal lpString As Any, ByVal lpFileName As String) As Long

'Usar estilos de XP
Declare Sub InitCommonControls Lib "comctl32" ()

'Obtener el nombre de la PC
Declare Function GetComputerName Lib "kernel32" Alias "GetComputerNameA" (ByVal lpBuffer As String, nSize As Long) As Long

'Establecer la impresora predeterminada de Windows
Public Declare Function SetDefaultPrinter Lib "winspool.drv" Alias "SetDefaultPrinterA" (ByVal pszPrinter As String) As Long

'Impedir el redibujado de formularios
Public Declare Function LockWindowUpdate Lib "user32" (ByVal hwndLock As Long) As Long

'Buscar una carpeta
Private Declare Sub CoTaskMemFree Lib "OLE32.DLL" (ByVal hMem As Long)
Private Declare Function lstrcat Lib "kernel32" Alias "lstrcatA" (ByVal lpString1 As String, ByVal lpString2 As String) As Long
Private Declare Function SHBrowseForFolder Lib "shell32" (lpbi As BrowseInfo) As Long

'Convertir una ruta devuelta por funciones como SHGetSpecialFolderLocation o SHBrowseForFolder
Private Declare Function SHGetPathFromIDList Lib "shell32.dll" Alias "SHGetPathFromIDListA" (ByVal pidl As Long, ByVal pszPath As String) As Long


'===============================================================================
'Declaracin de Subprocedimientos y Funciones Intermedias
'
'Subprocedimientos y Fuciones que invocan una funcin API
'===============================================================================


'Sub        : ListarUnidades(pcboLista As ComboBox)
'             Usa GetLogicalDrives para llenar un combo con las unidades disponibles.
'Parameters : pcboLista     Combo a llenar.

Public Sub ListarUnidades(pcboLista As ComboBox)
Dim lngLDs As Long
Dim i As Integer
Dim strDrv As String

    lngLDs = GetLogicalDrives

    For i = 0 To 25
        If (lngLDs And 2 ^ i) <> 0 Then
            strDrv = Chr$(65 + i)
            
            pcboLista.AddItem strDrv & ":" & " [" & DrvType(strDrv & ":\") & "]"
        End If
    Next
End Sub


'Function   : DrvType(pstrDrv As String) As String
'Parameters : pstrDrv   Volumen cuyo tipo se desea conocer (Formato U:\)
'Return     : Un string representando el tipo de unidad

Public Function DrvType(pstrDrv As String) As String
    Select Case GetDriveType(pstrDrv)
        Case 2
            'Si es un disco removible, asume que A: y B: sern _
            disqueteras, mientras que otras letras representarn _
            unidades Zip, Jazz o de otro tipo
            If UCase(pstrDrv) = "A:\" Or UCase(pstrDrv) = "B:\" Then
                DrvType = "Disquetera"
            Else
                DrvType = "Disco Removible"
            End If
        Case 3
            DrvType = "Disco Duro Local"
        Case 4
            DrvType = "Unidad de Red"
        Case 5
            DrvType = "Disco CD-ROM"
        Case 6
            DrvType = "RAM-Disk"
        Case 7
            DrvType = ""
    End Select
End Function


'Function   : ReadINI(pstrSection As String, pstrKey As String, pstrFile As String)
'             Llama a GetPrivateProfileString para leer un string en un archivo .ini.
'Parameters : pstrSection   Seccin
'             pstrKey       Clave
'             pstrFile      Nombre del archivo .INI
'Return     : El valor de la clave

Public Function ReadINI(pstrSection As String, pstrKey As String, pstrFile As String) As String
Dim strSeccion As String
Dim strClave As String
Dim strRet As String * 128
Dim lngLen As Long
    
    strSeccion = pstrSection + Chr(0)
    strClave = pstrKey + Chr(0)
    strRet = Space(128)
    
    lngLen = GetPrivateProfileString(strSeccion, strClave, "", strRet, 128, pstrFile)
    
    ReadINI = Left$(strRet, lngLen)
End Function


'Function   : WriteINI(pstrSection As String, pstrKey As String, pstrValue As String, pstrFile As String)
'             Llama a WritePrivateProfileString para escribir en un archivo .ini.
'Parameters : pstrSection   Seccin
'             pstrKey       Clave
'             pstrValue     Valor a escribir
'             pstrFile      Nombre del archivo .INI
'Return     : El valor de la clave

Public Function WriteINI(pstrSection As String, pstrKey As String, pstrValue As String, pstrFile As String)
    Call WritePrivateProfileString(pstrSection, pstrKey, pstrValue, pstrFile)
End Function


'Function   : GetPCName()
'             Llama a GetComputerName para obtener el nombre de la PC
'Parameters : None
'Return     : El nombre de la PC

Public Function GetPCName() As String
Dim strComputerName As String
Dim lngComputerNameLength As Long

    strComputerName = String(MAX_COMPUTERNAME_LENGTH + 1, 0)
    lngComputerNameLength = MAX_COMPUTERNAME_LENGTH
    Call GetComputerName(strComputerName, lngComputerNameLength)
    GetPCName = Mid(strComputerName, 1, lngComputerNameLength)
End Function


'----------------------------------------------------------------------------------
'Function   : BrowseForFolder(hwOwner As Long, pstrTitle As String, pstrInitialPath) As String
'Parameters : hwOwner           Manejador de la ventana propietaria
'             pstrTitle         texto en la parte superior de la ventana
'             pstrInitialPath   Carpeta en al que comienza el Browse
'Return     : La carpeta seleccionada o "" si se cancel el cuadro de dilogo
'----------------------------------------------------------------------------------

Public Function BrowseForFolder(hwOwner As Long, pstrTitle As String, pstrInitialPath) As String
Dim intNull As Integer
Dim lngIDList As Long
Dim lngResult As Long
Dim strPath As String
Dim udtBI As BrowseInfo

    With udtBI
        'Set the owner window
        .hwndOwner = hwOwner
        'lstrcat appends the two strings and returns the memory address
        .lpszTitle = lstrcat(pstrTitle, "")

        If Len(pstrInitialPath) Then
            ' Asignar la variable que contendr el directorio de inicio
            strFolderIni = pstrInitialPath
            ' Indicar la funcin Callback a usar.
            ' Como hay que asignar esa direccin a una variable,
            ' se usa una funcin "intermedia" que devuelve el valor
            ' del parmetro pasado... es decir: la direccin de la funcin!
            .lpfnCallback = rtnAddressOf(AddressOf BrowseFolderCallbackProc)
        End If
        
        'Return only if the user selected a directory
        .ulFlags = BIF_RETURNONLYFSDIRS
    End With

    'Show the 'Browse for folder' dialog
    lngIDList = SHBrowseForFolder(udtBI)
    If lngIDList Then
        strPath = String$(MAX_PATH, 0)
        'Get the path from the IDList
        SHGetPathFromIDList lngIDList, strPath
        'free the block of memory
        CoTaskMemFree lngIDList
        strPath = StripTerminator(strPath)
    End If

    BrowseForFolder = strPath
End Function


'----------------------------------------------------------------------------------
'Function   : BrowseFolderCallbackProc(ByVal hWndOwner As Long, ByVal uMSG As Long, ByVal lParam As Long, ByVal pData As Long) As Long
'
'Llamada CallBack para usar con la funcin BrowseForFolder
'
'ESTA FUNCIN FUE BAJADA DE LA PGINA DEL GUILLE
'http://guille.costasol.net/ejemplos/browsefolder.htm
'Y LIGERAMENTE RETOCADA PARA ADAPTARSE A MIS NECESIDADES
'----------------------------------------------------------------------------------

Function BrowseFolderCallbackProc(ByVal hwndOwner As Long, ByVal uMSG As Long, ByVal lParam As Long, ByVal pData As Long) As Long
Dim szDir As String

    On Local Error Resume Next

    If uMSG = BFFM_INITIALIZED Then
        ' Este mensaje se enviar cuando se inicia el dilogo,
        ' entonces es cuando hay que indicar el directorio de inicio.
        ' El path de inicio ser el directorio indicado,
        ' si no se ha asignado, usar el directorio actual
        If Len(strFolderIni) Then
            szDir = strFolderIni & Chr$(0)
        Else
            szDir = CurDir$ & Chr$(0)
        End If
        ' WParam  ser TRUE  si se especifica un path.
        '         ser FALSE si se especifica un pIDL.
        Call SendMessage(hwndOwner, BFFM_SETSELECTION, 1&, ByVal szDir)
    End If

    Err = 0
    BrowseFolderCallbackProc = 0
End Function


'----------------------------------------------------------------------------------
'Function   : rtnAddressOf(lngProc As Long) As Long
'
'Usado en la Llamada CallBack para la funcin BrowseForFolder
'
'ESTA FUNCIN FUE BAJADA DE LA PGINA DEL GUILLE
'http://guille.costasol.net/ejemplos/browsefolder.htm
'----------------------------------------------------------------------------------

Function rtnAddressOf(lngProc As Long) As Long
    ' Devuelve la direccin pasada como parmetro
    ' Esto se usar para asignar a una variable la direccin de una funcin
    ' o procedimiento.
    ' Por ejemplo, si en un tipo definido se asigna a una variable la direccin
    ' de una funcin o procedimiento
    rtnAddressOf = lngProc
End Function


'----------------------------------------------------------------------------------
'Function   : StripTerminator(pstrInput As String) As String
'Parameters : pstrInput Cadena de la cual se quieren extraer los Chr(0)'s
'Return     : Un string sin los Chr(0)'s
'----------------------------------------------------------------------------------

Public Function StripTerminator(pstrInput As String) As String
Dim intZeroPos As Integer
    'Search the position of the first chr$(0)
    intZeroPos = InStr(1, pstrInput, vbNullChar)
    
    If intZeroPos > 0 Then StripTerminator = Left$(pstrInput, intZeroPos - 1) Else StripTerminator = pstrInput
End Function
